#!/usr/bin/env Rscript

# Goal: identify and  mask_frameshift_alleles

  #working_dir <- "~/BfR/Projects/chewbbaca_frameshifts/Ringversuch/cgmlst_st001/list_3/cgmlst/P64_RV1_LC02_list_3A_ionS5"
  #working_dir <- "~/BfR/Projects/chewbbaca_frameshifts/Ringversuch/cgmlst_st001/list_3/cgmlst/P64_RV1_LC02_list_3A_ionS5"
  #working_dir <- "/home/carlus/BfR/Projects/chewbbaca_frameshifts/Ringversuch/spoiled_scheme/cgmlst/list_3/cgmlst/P64_RV1_LC02_list_3A_ionS5"
  
  # input:
  #sample_length_profile.file = file.path(working_dir,"allele_length.tsv")
  #scheme_length_profile.file = file.path(dirname(working_dir),"scheme_length_profile.tsv")
  #scheme_length_profile.file <- "/home/carlus/BfR/Projects/chewbbaca_frameshifts/pipeline_test/components/median_allele_length.tsv"
  #allele_profile.file = file.path(working_dir,"allele_profiles.tmp")
  
  sample_length_profile.file <- snakemake@input[["sample_length_profile"]]
  scheme_length_profile.file <- snakemake@input[["scheme_length_profile"]]
  allele_profile.file <- snakemake@input[["allele_profile"]]
  
  # output:
  #allele_profile_masked.file = file.path(working_dir,"allele_profiles.fsremoved.tsv")
  #masked_sites.file = file.path(working_dir,"masked_loci.tsv")
  
  allele_profile_masked.file <- snakemake@output[["allele_profile_masked"]]
  masked_sites.file <- snakemake@output[["masked_sites"]]

  # params:
    
  mode <- snakemake@params[["mode"]]
  threshold <- snakemake@params[["threshold"]]

  #mode = "relative" # absolute # TODO config config["parameters"]["frameshift_removal_mode"]
  #threshold = 0.01 # 0.01 # 10 config["parameters"]["frameshift_removal_threshold"]

  
  # load data -----
  
  sample_length_profile <- read.delim(sample_length_profile.file, stringsAsFactors = F)
  scheme_length_profile <- read.delim(scheme_length_profile.file, stringsAsFactors = F)
  
  
  allele_profile <- read.delim(allele_profile.file, stringsAsFactors = F)
  rownames(allele_profile) <- allele_profile[,1]
  allele_profile <- allele_profile[,-1]
  
  
  # find affected alleles ------
  locus_match <- match(sample_length_profile$locus,scheme_length_profile$locus)
  scheme_length_profile.matched <- scheme_length_profile[locus_match,]
  
  length_comparison <- data.frame(locus = as.character(scheme_length_profile.matched$locus), median_length = scheme_length_profile.matched$length, sample_length = sample_length_profile$length)
  length_comparison$diff <- abs(length_comparison$median_length - length_comparison$sample_length)
  length_comparison$ratio <- length_comparison$sample_length/length_comparison$median_length
  length_comparison$ratio2 <- abs(1- length_comparison$ratio)
  
  # remove NA
  length_comparison <- length_comparison[which(!is.na(length_comparison$diff)),]
  
  # find non na overlap
  length_comparison.selected <- length_comparison[length_comparison$diff != 0,]
  
  # filter profile -----
  if(mode == "relative"){
    length_comparison.outliers <- length_comparison.selected[length_comparison.selected$ratio2 > threshold  ,]
  } else {
    length_comparison.outliers <- length_comparison.selected[length_comparison.selected$diff > threshold  ,]
  }
  
  print(paste("Removing",nrow(length_comparison.outliers),"loci that deviate more than",threshold))
  
  loci2remove <- as.character(length_comparison.outliers$locus)
  
  
  allele_profile_masked <- allele_profile
  allele_profile_masked[,loci2remove] <- "-"
  
  # allele_profile_masked[,loci2remove]
  
  # export ------

  # prepare
  allele_profile_masked <- cbind(FILE=rownames(allele_profile_masked),allele_profile_masked)
  
  # write profile  
  write.table(allele_profile_masked,allele_profile_masked.file, sep = "\t", quote = F, row.names = F)
  
  # write masked loci table
  write.table(length_comparison.outliers, masked_sites.file, sep="\t", quote = F, row.names = F)

  