#!/usr/bin/env Rscript

# Goal: match external cluster names to internal cluster names


# define data ----

debug <- F
if (debug){
  
  # input
 # workdir <- "/home/carlus/BfR/Projects/salmonella-alleledb"
  #workdir <- "/cephfs/abteilung4/deneke/Projects/chewiesnake_validation/cgmlst_joineddatabase"
  workdir <- "/home/carlus/BfR/Projects/Pipeline_test/chewieSnake/chewiesnake_join/joined_db"
  #workdir <- "/cephfs/abteilung4/Projects_NGS/GenoSalmSurv/realtime_cgmlst/salmonella-alleledb"
#  external_cluster_names.file = "/home/carlus/BfR/Projects/salmonella-alleledb/RKI/epi_clusters.tsv" #config["parameters"]["external_cluster_names"], 

  #external_cluster_names.file = "/cephfs/abteilung4/Projects_NGS/GenoSalmSurv/realtime_cgmlst/test_BfR_dir/samples2serovars.tsv"
  #external_cluster_names.file = "/cephfs/abteilung4/Projects_NGS/GenoSalmSurv/realtime_cgmlst/salmonella-alleledb/RKI/epi_clusters.tsv" 
  #external_cluster_names.file = "/cephfs/abteilung4/Projects_NGS/GenoSalmSurv/realtime_cgmlst/test_BfR_dir/epi-sero_clusters.tsv"
  #external_cluster_names.file = "/cephfs/abteilung4/deneke/Projects/chewiesnake_validation/cgmlst_joineddatabase/external_cluster_names.tsv"
  external_cluster_names.file <- "/home/carlus/BfR/Projects/Pipeline_test/chewieSnake/chewiesnake_join/epiclusters.tsv"
  mapping_sample2clustername.file = file.path(workdir,"merged_db/mapping_sample2clustername.tsv")
  
  # output
  mapping_sample2externalclustername.file = file.path(workdir,"merged_db/mapping_sample2externalclustername.tsv")

} else {

  external_cluster_names.file = snakemake@input[["external_cluster_names"]]
  mapping_sample2clustername.file = snakemake@input[["mapping_sample2clustername_file"]]
  mapping_sample2externalclustername.file = snakemake@output[["mapping_sample2externalclustername_file"]]
  
  # parameters
  #ambiguity_handling
  
  # log
  log <- file(snakemake@log[[1]], open="wt")
  # redirect output to log file
  sink(log)
  sink(log, type="message")
  
}


# load data ----

if(!file.exists(external_cluster_names.file)) {stop("File",external_cluster_names.file,"does NOT exist")}
if(!file.exists(mapping_sample2clustername.file)) {stop("File",mapping_sample2clustername.file,"does NOT exist")}

external_cluster_names <- read.delim(external_cluster_names.file, stringsAsFactors = F, header = F, col.names = c("representative_sample","cluster_name") )
# remove line that has sample as first entry
if(nrow(external_cluster_names) > 0) if(external_cluster_names[1,1] == "sample") external_cluster_names <- external_cluster_names[-1,]
mapping_sample2clustername <- read.delim(mapping_sample2clustername.file, stringsAsFactors = F)

# checks ----

if (any(duplicated(external_cluster_names$representative_sample)) ){
  warning("Duplicated sample names found. sample associated to more than one cluster")
  #stop("Duplicated sample names found. sample associated to more than one cluster")
}

if (any(duplicated(external_cluster_names$cluster_name)) ){
  warning("Duplicated cluster names found. More than one sample associated to a cluster")
}

# execution ----

if(nrow(external_cluster_names) > 0){

  matched_external_clusters <- apply(external_cluster_names,1, function(external_cluster){
    
    match <- which(mapping_sample2clustername$sample == external_cluster["representative_sample"])
    if(length(match) == 0){
      warning("No match found for sample",external_cluster["representative_sample"],"from external cluster",external_cluster["cluster_name"])  
      return(NA)
    } else if(length(match) == 1){
      # match 
      matched_cluster <- mapping_sample2clustername$cluster_name[match]
      # find other matches in same cluster
      matched_data <- mapping_sample2clustername[mapping_sample2clustername$cluster_name == matched_cluster,]
      # assign external cluster name
      matched_data$external_cluster_name <- external_cluster["cluster_name"]
      # add reference sample name
      matched_data$representative_sample <- external_cluster["representative_sample"]
  
      return(matched_data)
    } else {
      stop("Matching of sample",external_cluster["representative_sample"],"was not unique.")
    }
    
  })
  
  # join to dataframe
  # handle zero  matches
  if(!is.list(matched_external_clusters)){
    matched_external_clusters <- as.data.frame(matrix(nrow=0, ncol=4))
    colnames(matched_external_clusters) <- c("sample","cluster_name","external_cluster_name","representative_sample")  
  } else {
    matched_external_clusters <- do.call(rbind,matched_external_clusters)  
  }
  

  # remove NAs
  matched_external_clusters <- matched_external_clusters[!is.na(matched_external_clusters$sample),]
  
  # multi-match handling
  matched_external_clusters <- do.call(rbind,lapply(unique(matched_external_clusters$sample), function(mysample){
    mymatch <- matched_external_clusters[matched_external_clusters$sample == mysample,]
    
    if(nrow(mymatch) > 1){
      return(data.frame(sample=mysample,
                 cluster_name=paste(unique(mymatch$cluster_name), collapse = ";"),
                 external_cluster_name = paste(unique(mymatch$external_cluster_name), collapse = ";"), # join all external cluster names
                 representative_sample = unique(mymatch$representative_sample)[1]) ) # keep single represenative
    } else {
      return(mymatch)
    }
  }))
  
  # handle zero  matches
  if(!is.list(matched_external_clusters)){
    matched_external_clusters <- as.data.frame(matrix(nrow=0, ncol=4))
    colnames(matched_external_clusters) <- c("sample","cluster_name","external_cluster_name","representative_sample")  
  }
  
} else {
  print("Exception: external cluster name list is emtpy")
  
  matched_external_clusters <- matrix(nrow=0, ncol=4)
  colnames(matched_external_clusters) <- c("sample","cluster_name",    "external_cluster_name","representative_sample")
  
}
# write out
write.table(matched_external_clusters, file = mapping_sample2externalclustername.file, quote = F, row.names = F, sep = "\t")

print(paste("Success. Output written to",mapping_sample2externalclustername.file))
