# subclustering cluster files

# libraries -----------------------------------

#suppressPackageStartupMessages(library(foreach))
suppressPackageStartupMessages(library(magrittr))
suppressPackageStartupMessages(library(dendextend))
suppressPackageStartupMessages(library(reshape2)) # for converting matrix
suppressPackageStartupMessages(library(dplyr))


# hierarchically ? 


# define data

debug <- F # T # F
if(debug){

  workdir <- "/home/carlus/BfR/Projects/salmonella-alleledb"
#  workdir <- "/cephfs/abteilung4/Projects_NGS/GenoSalmSurv/realtime_cgmlst/salmonella-alleledb"
  cluster <- "Palaephatus" #"Mnaseas"
  samples_in_cluster.file <- file.path(workdir, "merged_db","clustering",paste0("samples_",cluster,".tsv"))

  # input
  distance_matrix_cluster_tsv = file.path(workdir,"merged_db/clustering",cluster,"distance_matrix.tsv")
  # mapping_sample2clustername_file = "merged_db/clustering/{i}/mapping_sample2subclustername.tsv"

  #output:
  subclusters.file = file.path(workdir,"merged_db/clustering",cluster,"mapping_sample2subclustername.tsv")

  # params:

  mapping_sample2clustername_file = file.path(workdir,"merged_db/clustering",cluster,"mapping_sample2subclustername.tsv")
  distance_thresholds = 3 # c(10,3)
  name_types = "greek-alphabet" # c("numerical","greek-alphabet")
  clustering_method = "single"
  functions_file <- "functions.R"
  
  greekletters_file <- file.path("greek_letters.txt")
  
} else {
  
  # snakemake input
  distance_matrix_cluster_tsv <- snakemake@input[["distance_matrix_cluster_tsv"]]
  # output
  subclusters.file  <- snakemake@output[["subclusters"]]
  # params
  mapping_sample2clustername_file <- snakemake@params[["mapping_sample2clustername_file"]]
  distance_thresholds <- snakemake@params[["distance_thresholds"]]
  name_types <- snakemake@params[["name_types"]]
  clustering_method <- snakemake@params[["clustering_method"]]

  
  functions_file <- file.path(snakemake@params[["functions_file"]],"scripts/functions.R")
  
  greekletters_file <- file.path(snakemake@params[["functions_file"]],"scripts/greek_letters.txt")
  
  
  log <- file(snakemake@log[[1]], open="wt")
  
  # redirect output to log file
  sink(log)
  sink(log, type="message")
  
}


# load functions ----
if(! file.exists(functions_file)) stop("Function file",functions_file,"does NOT exist")
source(functions_file)

# load data

# distance matrix of cluster
distance_matrix <- read_distance_matrix(distance_matrix_cluster_tsv, sep="\t", header = T)

# load nomenclature
if(! file.exists(mapping_sample2clustername_file)) {
  warning("File",mapping_sample2clustername_file,"does NOT exist")
  mapping_sample2clustername_all <- data.frame(sample=character(), stringsAsFactors=FALSE) 
} else {
  mapping_sample2clustername_loaded <- read.delim(file = mapping_sample2clustername_file, sep="\t", stringsAsFactors = F, header = T, colClasses = "character")
  mapping_sample2clustername_all <- mapping_sample2clustername_loaded
}


# load potential names
# if(!file.exists(potential_names_file)) stop("File with potential cluster names",potential_names_file,"does NOT exist\nExpected a text file with one name per row")
#potential_clusternames <- readLines(potential_names_file)

potential_clusternames_numerical <- as.character(c(1:99))
if(!file.exists(greekletters_file)) stop("File",greekletters_file,"does NOT exist")
potential_clusternames_greekalphabet <- readLines(greekletters_file)

print("All files loaded")

# execute ---

# cluster
clustering <- distance_matrix %>% as.dist %>% hclust(., method = clustering_method) %>% as.dendrogram

#mapping_sample2clustername_list <- foreach(i = 1:length(distance_thresholds)) %do% { # for each distance threshold
  
mapping_sample2clustername_list <- lapply(1:length(distance_thresholds), function(i) { # for each distance threshold  
  
  distance_threshold <- distance_thresholds[i]
  print(paste("Computing cluster code for threshold",distance_threshold))
  
  mycolname <- paste("cluster_name",distance_threshold,sep = "_")
  name_type <- name_types[i]

  if (is.na(name_type)){
    warning("Setting potential_clusternames to numerical")
    potential_clusternames <- potential_clusternames_numerical
  } else if (name_type == "numerical"){
    potential_clusternames <- potential_clusternames_numerical
  #} else if(name_type == "greek-alphabet") {
  } else if(grepl("greek",name_type, ignore.case = T)) {
    potential_clusternames <- potential_clusternames_greekalphabet
  } else {
    warning("Setting potential_clusternames to numerical")
    potential_clusternames <- potential_clusternames_numerical
  }

  
  # cluster numbers
  samples2cluster <- Get_samples2cluster(dendro = clustering, cut_threshold = distance_threshold, removeOrphans = F) # do keep orphans here
  #samples2cluster <- Get_samples2cluster(dendro = clustering, cut_threshold = distance_threshold, removeOrphans = T) # remove orphans

  # match samples to existing cluster names
  matched_column <- which(colnames(mapping_sample2clustername_all) == mycolname)
  if(length(matched_column) == 1) { # data already found for this distance
    print("data already found for this distance")
    mapping_sample2clustername.local <- mapping_sample2clustername_all[c("sample",mycolname)]
    colnames(mapping_sample2clustername.local) <- c("sample","cluster_name")
  } else {  # create empty list
    print("create empty list")
    mapping_sample2clustername.local <- data.frame(sample=character(), cluster_name=character(), stringsAsFactors=FALSE) 
  }

  # merge  
  samples2clustername <- merge(x=samples2cluster,y=mapping_sample2clustername.local, by.x="sample", by.y = "sample", all.x = T, all.y = F)

  # find all unique cluster_number to cluster_name pairs
  clusternumber2name <- samples2clustername[,c("cluster_number","cluster_name")] %>% filter(!is.na(cluster_name)) %>% unique

  # check cluster merge and split conflicts
  check_clusternaming_conflicts(clusternumber2name)
  
  # add new samples with same cluster number
  update_results <- update_samplenames (samples2clustername, clusternumber2name, verbose=T, potential_clusternames = potential_clusternames)
  samples2clustername <- update_results$samples2clustername
  clusternumber2name <- update_results$clusternumber2name
  
  # extract cluster name and rename column name
  mapping_sample2clustername_updated <- samples2clustername[,c("sample","cluster_name")]
  colnames(mapping_sample2clustername_updated) <- c("sample", mycolname)
  
  return(mapping_sample2clustername_updated)
})


# merge individual files

mapping_sample2clustername_merged <- Reduce(function(...) merge(..., by="sample", all=TRUE), mapping_sample2clustername_list)

# export

write.table(mapping_sample2clustername_merged, file = subclusters.file, sep="\t", quote = F, row.names = F)

# --------------

print("Done")

